"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingmodel import BillingModel
from .billingtier import BillingTier
from .dto_createpricetier import DtoCreatePriceTier, DtoCreatePriceTierTypedDict
from .price_transformquantity import (
    PriceTransformQuantity,
    PriceTransformQuantityTypedDict,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoUpdatePriceRequestTypedDict(TypedDict):
    amount: NotRequired[str]
    r"""Amount is the new price amount that overrides the original price (optional)"""
    billing_model: NotRequired[BillingModel]
    description: NotRequired[str]
    display_name: NotRequired[str]
    effective_from: NotRequired[str]
    group_id: NotRequired[str]
    r"""GroupID is the id of the group to update the price in"""
    lookup_key: NotRequired[str]
    r"""All price fields that can be updated
    Non-critical fields (can be updated directly)
    """
    metadata: NotRequired[Dict[str, str]]
    price_unit_amount: NotRequired[str]
    r"""PriceUnitAmount is the price unit amount (for CUSTOM price unit type, FLAT_FEE/PACKAGE billing models)"""
    price_unit_tiers: NotRequired[List[DtoCreatePriceTierTypedDict]]
    r"""PriceUnitTiers are the price unit tiers (for CUSTOM price unit type, TIERED billing model)"""
    tier_mode: NotRequired[BillingTier]
    tiers: NotRequired[List[DtoCreatePriceTierTypedDict]]
    r"""Tiers determines the pricing tiers for this line item"""
    transform_quantity: NotRequired[PriceTransformQuantityTypedDict]


class DtoUpdatePriceRequest(BaseModel):
    amount: Optional[str] = None
    r"""Amount is the new price amount that overrides the original price (optional)"""

    billing_model: Optional[BillingModel] = None

    description: Optional[str] = None

    display_name: Optional[str] = None

    effective_from: Optional[str] = None

    group_id: Optional[str] = None
    r"""GroupID is the id of the group to update the price in"""

    lookup_key: Optional[str] = None
    r"""All price fields that can be updated
    Non-critical fields (can be updated directly)
    """

    metadata: Optional[Dict[str, str]] = None

    price_unit_amount: Optional[str] = None
    r"""PriceUnitAmount is the price unit amount (for CUSTOM price unit type, FLAT_FEE/PACKAGE billing models)"""

    price_unit_tiers: Optional[List[DtoCreatePriceTier]] = None
    r"""PriceUnitTiers are the price unit tiers (for CUSTOM price unit type, TIERED billing model)"""

    tier_mode: Optional[BillingTier] = None

    tiers: Optional[List[DtoCreatePriceTier]] = None
    r"""Tiers determines the pricing tiers for this line item"""

    transform_quantity: Optional[PriceTransformQuantity] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "billing_model",
                "description",
                "display_name",
                "effective_from",
                "group_id",
                "lookup_key",
                "metadata",
                "price_unit_amount",
                "price_unit_tiers",
                "tier_mode",
                "tiers",
                "transform_quantity",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcadence import BillingCadence
from .billingperiod import BillingPeriod
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoBillingCycleInfoTypedDict(TypedDict):
    billing_anchor: NotRequired[str]
    r"""billing_anchor is the new billing anchor"""
    billing_cadence: NotRequired[BillingCadence]
    billing_period: NotRequired[BillingPeriod]
    billing_period_count: NotRequired[int]
    r"""billing_period_count is the billing period count"""
    period_end: NotRequired[str]
    r"""period_end is the end of the new billing period"""
    period_start: NotRequired[str]
    r"""period_start is the start of the new billing period"""


class DtoBillingCycleInfo(BaseModel):
    billing_anchor: Optional[str] = None
    r"""billing_anchor is the new billing anchor"""

    billing_cadence: Optional[BillingCadence] = None

    billing_period: Optional[BillingPeriod] = None

    billing_period_count: Optional[int] = None
    r"""billing_period_count is the billing period count"""

    period_end: Optional[str] = None
    r"""period_end is the end of the new billing period"""

    period_start: Optional[str] = None
    r"""period_start is the start of the new billing period"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "billing_anchor",
                "billing_cadence",
                "billing_period",
                "billing_period_count",
                "period_end",
                "period_start",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_featureresponse import DtoFeatureResponse, DtoFeatureResponseTypedDict
from .entitlemententitytype import EntitlementEntityType
from .entitlementusageresetperiod import EntitlementUsageResetPeriod
from .featuretype import FeatureType
from .status import Status
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional, TYPE_CHECKING
from typing_extensions import NotRequired, TypedDict

if TYPE_CHECKING:
    from .dto_addonresponse import DtoAddonResponse, DtoAddonResponseTypedDict
    from .dto_planresponse import DtoPlanResponse, DtoPlanResponseTypedDict


class DtoEntitlementResponseTypedDict(TypedDict):
    addon: NotRequired["DtoAddonResponseTypedDict"]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    display_order: NotRequired[int]
    end_date: NotRequired[str]
    entity_id: NotRequired[str]
    entity_type: NotRequired[EntitlementEntityType]
    environment_id: NotRequired[str]
    feature: NotRequired[DtoFeatureResponseTypedDict]
    feature_id: NotRequired[str]
    feature_type: NotRequired[FeatureType]
    id: NotRequired[str]
    is_enabled: NotRequired[bool]
    is_soft_limit: NotRequired[bool]
    parent_entitlement_id: NotRequired[str]
    plan: NotRequired["DtoPlanResponseTypedDict"]
    plan_id: NotRequired[str]
    r"""TODO: Remove this once we have a proper entitlement entity type"""
    start_date: NotRequired[str]
    static_value: NotRequired[str]
    status: NotRequired[Status]
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]
    usage_limit: NotRequired[int]
    usage_reset_period: NotRequired[EntitlementUsageResetPeriod]


class DtoEntitlementResponse(BaseModel):
    addon: Optional["DtoAddonResponse"] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    display_order: Optional[int] = None

    end_date: Optional[str] = None

    entity_id: Optional[str] = None

    entity_type: Optional[EntitlementEntityType] = None

    environment_id: Optional[str] = None

    feature: Optional[DtoFeatureResponse] = None

    feature_id: Optional[str] = None

    feature_type: Optional[FeatureType] = None

    id: Optional[str] = None

    is_enabled: Optional[bool] = None

    is_soft_limit: Optional[bool] = None

    parent_entitlement_id: Optional[str] = None

    plan: Optional["DtoPlanResponse"] = None

    plan_id: Optional[str] = None
    r"""TODO: Remove this once we have a proper entitlement entity type"""

    start_date: Optional[str] = None

    static_value: Optional[str] = None

    status: Optional[Status] = None

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    usage_limit: Optional[int] = None

    usage_reset_period: Optional[EntitlementUsageResetPeriod] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "addon",
                "created_at",
                "created_by",
                "display_order",
                "end_date",
                "entity_id",
                "entity_type",
                "environment_id",
                "feature",
                "feature_id",
                "feature_type",
                "id",
                "is_enabled",
                "is_soft_limit",
                "parent_entitlement_id",
                "plan",
                "plan_id",
                "start_date",
                "static_value",
                "status",
                "tenant_id",
                "updated_at",
                "updated_by",
                "usage_limit",
                "usage_reset_period",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_createpricerequest import (
    DtoCreatePriceRequest,
    DtoCreatePriceRequestTypedDict,
)
from flexprice_py.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class DtoCreateBulkPriceRequestTypedDict(TypedDict):
    items: List[DtoCreatePriceRequestTypedDict]


class DtoCreateBulkPriceRequest(BaseModel):
    items: List[DtoCreatePriceRequest]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoInvoiceLineItemPreviewTypedDict(TypedDict):
    amount: NotRequired[str]
    r"""amount for this line item"""
    description: NotRequired[str]
    r"""description of the line item"""
    is_proration: NotRequired[bool]
    r"""is_proration indicates if this line item is a proration"""
    period_end: NotRequired[str]
    r"""period_end for this line item (if applicable)"""
    period_start: NotRequired[str]
    r"""period_start for this line item (if applicable)"""
    quantity: NotRequired[str]
    r"""quantity for this line item"""
    unit_price: NotRequired[str]
    r"""unit_price for this line item"""


class DtoInvoiceLineItemPreview(BaseModel):
    amount: Optional[str] = None
    r"""amount for this line item"""

    description: Optional[str] = None
    r"""description of the line item"""

    is_proration: Optional[bool] = None
    r"""is_proration indicates if this line item is a proration"""

    period_end: Optional[str] = None
    r"""period_end for this line item (if applicable)"""

    period_start: Optional[str] = None
    r"""period_start for this line item (if applicable)"""

    quantity: Optional[str] = None
    r"""quantity for this line item"""

    unit_price: Optional[str] = None
    r"""unit_price for this line item"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "description",
                "is_proration",
                "period_end",
                "period_start",
                "quantity",
                "unit_price",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreatePriceUnitRequestTypedDict(TypedDict):
    base_currency: str
    r"""base_currency  is the currency that the price unit is based on"""
    code: str
    conversion_rate: str
    r"""ConversionRate defines the exchange rate from this price unit to the base currency.
    This rate is used to convert amounts in the custom price unit to the base currency for storage and billing.

    Conversion formula:
    price_unit_amount * conversion_rate = base_currency_amount

    Example:
    If conversion_rate = \"0.01\" and base_currency = \"usd\":
    100 price_unit tokens * 0.01 = 1.00 USD

    Note: Rounding precision is determined by the base currency (e.g., USD uses 2 decimal places, JPY uses 0).
    """
    name: str
    symbol: str
    metadata: NotRequired[Dict[str, str]]


class DtoCreatePriceUnitRequest(BaseModel):
    base_currency: str
    r"""base_currency  is the currency that the price unit is based on"""

    code: str

    conversion_rate: str
    r"""ConversionRate defines the exchange rate from this price unit to the base currency.
    This rate is used to convert amounts in the custom price unit to the base currency for storage and billing.

    Conversion formula:
    price_unit_amount * conversion_rate = base_currency_amount

    Example:
    If conversion_rate = \"0.01\" and base_currency = \"usd\":
    100 price_unit tokens * 0.01 = 1.00 USD

    Note: Rounding precision is determined by the base currency (e.g., USD uses 2 decimal places, JPY uses 0).
    """

    name: str

    symbol: str

    metadata: Optional[Dict[str, str]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .creditgrantcadence import CreditGrantCadence
from .creditgrantexpirydurationunit import CreditGrantExpiryDurationUnit
from .creditgrantexpirytype import CreditGrantExpiryType
from .creditgrantperiod import CreditGrantPeriod
from .creditgrantscope import CreditGrantScope
from .status import Status
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreditGrantResponseTypedDict(TypedDict):
    cadence: NotRequired[CreditGrantCadence]
    conversion_rate: NotRequired[str]
    r"""amount in the currency =  number of credits * conversion_rate
    ex if conversion_rate is 1, then 1 USD = 1 credit
    ex if conversion_rate is 2, then 1 USD = 0.5 credits
    ex if conversion_rate is 0.5, then 1 USD = 2 credits
    """
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    credit_grant_anchor: NotRequired[str]
    credits: NotRequired[str]
    end_date: NotRequired[str]
    environment_id: NotRequired[str]
    expiration_duration: NotRequired[int]
    expiration_duration_unit: NotRequired[CreditGrantExpiryDurationUnit]
    expiration_type: NotRequired[CreditGrantExpiryType]
    id: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    name: NotRequired[str]
    period: NotRequired[CreditGrantPeriod]
    period_count: NotRequired[int]
    plan_id: NotRequired[str]
    priority: NotRequired[int]
    scope: NotRequired[CreditGrantScope]
    start_date: NotRequired[str]
    status: NotRequired[Status]
    subscription_id: NotRequired[str]
    tenant_id: NotRequired[str]
    topup_conversion_rate: NotRequired[str]
    r"""topup_conversion_rate is the conversion rate for the topup to the currency
    ex if topup_conversion_rate is 1, then 1 USD = 1 credit
    ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
    ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
    """
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoCreditGrantResponse(BaseModel):
    cadence: Optional[CreditGrantCadence] = None

    conversion_rate: Optional[str] = None
    r"""amount in the currency =  number of credits * conversion_rate
    ex if conversion_rate is 1, then 1 USD = 1 credit
    ex if conversion_rate is 2, then 1 USD = 0.5 credits
    ex if conversion_rate is 0.5, then 1 USD = 2 credits
    """

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    credit_grant_anchor: Optional[str] = None

    credits: Optional[str] = None

    end_date: Optional[str] = None

    environment_id: Optional[str] = None

    expiration_duration: Optional[int] = None

    expiration_duration_unit: Optional[CreditGrantExpiryDurationUnit] = None

    expiration_type: Optional[CreditGrantExpiryType] = None

    id: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    name: Optional[str] = None

    period: Optional[CreditGrantPeriod] = None

    period_count: Optional[int] = None

    plan_id: Optional[str] = None

    priority: Optional[int] = None

    scope: Optional[CreditGrantScope] = None

    start_date: Optional[str] = None

    status: Optional[Status] = None

    subscription_id: Optional[str] = None

    tenant_id: Optional[str] = None

    topup_conversion_rate: Optional[str] = None
    r"""topup_conversion_rate is the conversion rate for the topup to the currency
    ex if topup_conversion_rate is 1, then 1 USD = 1 credit
    ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
    ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
    """

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "cadence",
                "conversion_rate",
                "created_at",
                "created_by",
                "credit_grant_anchor",
                "credits",
                "end_date",
                "environment_id",
                "expiration_duration",
                "expiration_duration_unit",
                "expiration_type",
                "id",
                "metadata",
                "name",
                "period",
                "period_count",
                "plan_id",
                "priority",
                "scope",
                "start_date",
                "status",
                "subscription_id",
                "tenant_id",
                "topup_conversion_rate",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, Mapping, Optional


class Webhooks(BaseSDK):
    def handle_chargebee_webhook(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Chargebee webhook events

        Use as the Chargebee webhook endpoint URL. Receives payment and subscription events from Chargebee to sync status into FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleChargebeeWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
        )

        req = self._build_request(
            method="POST",
            path="/webhooks/chargebee/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleChargebeeWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, ["400", "401", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, ["500", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def handle_chargebee_webhook_async(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Chargebee webhook events

        Use as the Chargebee webhook endpoint URL. Receives payment and subscription events from Chargebee to sync status into FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleChargebeeWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/webhooks/chargebee/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleChargebeeWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, ["400", "401", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, ["500", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def handle_hubspot_webhook(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        x_hub_spot_signature_v3: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle HubSpot webhook events

        Use as the HubSpot webhook endpoint URL. Receives deal and customer events (e.g. deal closed won) to create or update customers in FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param x_hub_spot_signature_v3: HubSpot webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleHubspotWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            x_hub_spot_signature_v3=x_hub_spot_signature_v3,
        )

        req = self._build_request(
            method="POST",
            path="/webhooks/hubspot/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleHubspotWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def handle_hubspot_webhook_async(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        x_hub_spot_signature_v3: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle HubSpot webhook events

        Use as the HubSpot webhook endpoint URL. Receives deal and customer events (e.g. deal closed won) to create or update customers in FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param x_hub_spot_signature_v3: HubSpot webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleHubspotWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            x_hub_spot_signature_v3=x_hub_spot_signature_v3,
        )

        req = self._build_request_async(
            method="POST",
            path="/webhooks/hubspot/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleHubspotWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def handle_moyasar_webhook(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        x_moyasar_signature: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Moyasar webhook events

        Use as the Moyasar webhook endpoint URL. Receives payment events from Moyasar to update payment status in FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param x_moyasar_signature: Moyasar webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleMoyasarWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            x_moyasar_signature=x_moyasar_signature,
        )

        req = self._build_request(
            method="POST",
            path="/webhooks/moyasar/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleMoyasarWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def handle_moyasar_webhook_async(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        x_moyasar_signature: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Moyasar webhook events

        Use as the Moyasar webhook endpoint URL. Receives payment events from Moyasar to update payment status in FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param x_moyasar_signature: Moyasar webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleMoyasarWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            x_moyasar_signature=x_moyasar_signature,
        )

        req = self._build_request_async(
            method="POST",
            path="/webhooks/moyasar/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleMoyasarWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def handle_nomod_webhook(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        x_api_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Nomod webhook events

        Use as the Nomod webhook endpoint URL. Receives payment and invoice events from Nomod to keep FlexPrice in sync.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param x_api_key: Nomod webhook secret (if configured)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleNomodWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            x_api_key=x_api_key,
        )

        req = self._build_request(
            method="POST",
            path="/webhooks/nomod/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleNomodWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, ["401", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def handle_nomod_webhook_async(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        x_api_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Nomod webhook events

        Use as the Nomod webhook endpoint URL. Receives payment and invoice events from Nomod to keep FlexPrice in sync.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param x_api_key: Nomod webhook secret (if configured)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleNomodWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            x_api_key=x_api_key,
        )

        req = self._build_request_async(
            method="POST",
            path="/webhooks/nomod/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleNomodWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, ["401", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def handle_quickbooks_webhook(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        intuit_signature: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle QuickBooks webhook events

        Use as the QuickBooks webhook endpoint URL. Receives payment events from QuickBooks to sync payment status into FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param intuit_signature: QuickBooks webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleQuickbooksWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            intuit_signature=intuit_signature,
        )

        req = self._build_request(
            method="POST",
            path="/webhooks/quickbooks/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleQuickbooksWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, ["400", "401", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, ["500", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def handle_quickbooks_webhook_async(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        intuit_signature: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle QuickBooks webhook events

        Use as the QuickBooks webhook endpoint URL. Receives payment events from QuickBooks to sync payment status into FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param intuit_signature: QuickBooks webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleQuickbooksWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            intuit_signature=intuit_signature,
        )

        req = self._build_request_async(
            method="POST",
            path="/webhooks/quickbooks/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleQuickbooksWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, ["400", "401", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, ["500", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def handle_razorpay_webhook(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        x_razorpay_signature: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Razorpay webhook events

        Use as the Razorpay webhook endpoint URL. Receives payment capture and failure events to update invoice or payment status in FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param x_razorpay_signature: Razorpay webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleRazorpayWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            x_razorpay_signature=x_razorpay_signature,
        )

        req = self._build_request(
            method="POST",
            path="/webhooks/razorpay/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleRazorpayWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def handle_razorpay_webhook_async(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        x_razorpay_signature: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Razorpay webhook events

        Use as the Razorpay webhook endpoint URL. Receives payment capture and failure events to update invoice or payment status in FlexPrice.

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param x_razorpay_signature: Razorpay webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleRazorpayWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            x_razorpay_signature=x_razorpay_signature,
        )

        req = self._build_request_async(
            method="POST",
            path="/webhooks/razorpay/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleRazorpayWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def handle_stripe_webhook(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        stripe_signature: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Stripe webhook events

        Use as the Stripe webhook endpoint URL. Receives payment and customer events from Stripe to keep FlexPrice in sync (e.g. payment succeeded, customer created).

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param stripe_signature: Stripe webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleStripeWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            stripe_signature=stripe_signature,
        )

        req = self._build_request(
            method="POST",
            path="/webhooks/stripe/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleStripeWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, ["400", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, ["500", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def handle_stripe_webhook_async(
        self,
        *,
        tenant_id: str,
        environment_id: str,
        stripe_signature: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Handle Stripe webhook events

        Use as the Stripe webhook endpoint URL. Receives payment and customer events from Stripe to keep FlexPrice in sync (e.g. payment succeeded, customer created).

        :param tenant_id: Tenant ID
        :param environment_id: Environment ID
        :param stripe_signature: Stripe webhook signature
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.HandleStripeWebhookRequest(
            tenant_id=tenant_id,
            environment_id=environment_id,
            stripe_signature=stripe_signature,
        )

        req = self._build_request_async(
            method="POST",
            path="/webhooks/stripe/{tenant_id}/{environment_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="handleStripeWebhook",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, ["400", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, ["500", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .commitmentinfo import CommitmentInfo, CommitmentInfoTypedDict
from .dto_sourceusageitem import DtoSourceUsageItem, DtoSourceUsageItemTypedDict
from .dto_usagebreakdownitem import (
    DtoUsageBreakdownItem,
    DtoUsageBreakdownItemTypedDict,
)
from .status import Status
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoInvoiceLineItemResponseTypedDict(TypedDict):
    amount: NotRequired[str]
    commitment_info: NotRequired[CommitmentInfoTypedDict]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    currency: NotRequired[str]
    customer_id: NotRequired[str]
    display_name: NotRequired[str]
    entity_id: NotRequired[str]
    entity_type: NotRequired[str]
    environment_id: NotRequired[str]
    id: NotRequired[str]
    invoice_id: NotRequired[str]
    invoice_level_discount: NotRequired[str]
    r"""invoice_level_discount is the discount amount in invoice currency applied to all line items on the invoice."""
    line_item_discount: NotRequired[str]
    r"""line_item_discount is the discount amount in invoice currency applied directly to this line item."""
    metadata: NotRequired[Dict[str, str]]
    meter_display_name: NotRequired[str]
    meter_id: NotRequired[str]
    period_end: NotRequired[str]
    period_start: NotRequired[str]
    plan_display_name: NotRequired[str]
    prepaid_credits_applied: NotRequired[str]
    r"""prepaid_credits_applied is the amount in invoice currency reduced from this line item due to prepaid credits application."""
    price_id: NotRequired[str]
    price_type: NotRequired[str]
    price_unit: NotRequired[str]
    price_unit_amount: NotRequired[str]
    price_unit_id: NotRequired[str]
    quantity: NotRequired[str]
    status: NotRequired[Status]
    subscription_id: NotRequired[str]
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]
    usage_analytics: NotRequired[List[DtoSourceUsageItemTypedDict]]
    r"""usage_analytics contains usage analytics for this line item (legacy - grouped by source)"""
    usage_breakdown: NotRequired[List[DtoUsageBreakdownItemTypedDict]]
    r"""usage_breakdown contains flexible usage breakdown for this line item (supports any grouping)"""


class DtoInvoiceLineItemResponse(BaseModel):
    amount: Optional[str] = None

    commitment_info: Optional[CommitmentInfo] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    currency: Optional[str] = None

    customer_id: Optional[str] = None

    display_name: Optional[str] = None

    entity_id: Optional[str] = None

    entity_type: Optional[str] = None

    environment_id: Optional[str] = None

    id: Optional[str] = None

    invoice_id: Optional[str] = None

    invoice_level_discount: Optional[str] = None
    r"""invoice_level_discount is the discount amount in invoice currency applied to all line items on the invoice."""

    line_item_discount: Optional[str] = None
    r"""line_item_discount is the discount amount in invoice currency applied directly to this line item."""

    metadata: Optional[Dict[str, str]] = None

    meter_display_name: Optional[str] = None

    meter_id: Optional[str] = None

    period_end: Optional[str] = None

    period_start: Optional[str] = None

    plan_display_name: Optional[str] = None

    prepaid_credits_applied: Optional[str] = None
    r"""prepaid_credits_applied is the amount in invoice currency reduced from this line item due to prepaid credits application."""

    price_id: Optional[str] = None

    price_type: Optional[str] = None

    price_unit: Optional[str] = None

    price_unit_amount: Optional[str] = None

    price_unit_id: Optional[str] = None

    quantity: Optional[str] = None

    status: Optional[Status] = None

    subscription_id: Optional[str] = None

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    usage_analytics: Optional[List[DtoSourceUsageItem]] = None
    r"""usage_analytics contains usage analytics for this line item (legacy - grouped by source)"""

    usage_breakdown: Optional[List[DtoUsageBreakdownItem]] = None
    r"""usage_breakdown contains flexible usage breakdown for this line item (supports any grouping)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "commitment_info",
                "created_at",
                "created_by",
                "currency",
                "customer_id",
                "display_name",
                "entity_id",
                "entity_type",
                "environment_id",
                "id",
                "invoice_id",
                "invoice_level_discount",
                "line_item_discount",
                "metadata",
                "meter_display_name",
                "meter_id",
                "period_end",
                "period_start",
                "plan_display_name",
                "prepaid_credits_applied",
                "price_id",
                "price_type",
                "price_unit",
                "price_unit_amount",
                "price_unit_id",
                "quantity",
                "status",
                "subscription_id",
                "tenant_id",
                "updated_at",
                "updated_by",
                "usage_analytics",
                "usage_breakdown",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoProrationDetailsTypedDict(TypedDict):
    charge_amount: NotRequired[str]
    r"""charge_amount is the charge amount for the new subscription"""
    charge_description: NotRequired[str]
    r"""charge_description describes what the charge is for"""
    credit_amount: NotRequired[str]
    r"""credit_amount is the credit amount from the old subscription"""
    credit_description: NotRequired[str]
    r"""credit_description describes what the credit is for"""
    currency: NotRequired[str]
    r"""currency is the currency for all amounts"""
    current_period_end: NotRequired[str]
    r"""current_period_end is the end of the current billing period"""
    current_period_start: NotRequired[str]
    r"""current_period_start is the start of the current billing period"""
    days_remaining: NotRequired[int]
    r"""days_remaining is the number of days remaining in the current period"""
    days_used: NotRequired[int]
    r"""days_used is the number of days used in the current period"""
    net_amount: NotRequired[str]
    r"""net_amount is the net amount (charge - credit)"""
    proration_date: NotRequired[str]
    r"""proration_date is the date used for proration calculations"""


class DtoProrationDetails(BaseModel):
    charge_amount: Optional[str] = None
    r"""charge_amount is the charge amount for the new subscription"""

    charge_description: Optional[str] = None
    r"""charge_description describes what the charge is for"""

    credit_amount: Optional[str] = None
    r"""credit_amount is the credit amount from the old subscription"""

    credit_description: Optional[str] = None
    r"""credit_description describes what the credit is for"""

    currency: Optional[str] = None
    r"""currency is the currency for all amounts"""

    current_period_end: Optional[str] = None
    r"""current_period_end is the end of the current billing period"""

    current_period_start: Optional[str] = None
    r"""current_period_start is the start of the current billing period"""

    days_remaining: Optional[int] = None
    r"""days_remaining is the number of days remaining in the current period"""

    days_used: Optional[int] = None
    r"""days_used is the number of days used in the current period"""

    net_amount: Optional[str] = None
    r"""net_amount is the net amount (charge - credit)"""

    proration_date: Optional[str] = None
    r"""proration_date is the date used for proration calculations"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "charge_amount",
                "charge_description",
                "credit_amount",
                "credit_description",
                "currency",
                "current_period_end",
                "current_period_start",
                "days_remaining",
                "days_used",
                "net_amount",
                "proration_date",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


SecretType = Union[
    Literal[
        "private_key",
        "publishable_key",
        "integration",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .s3jobconfig import S3JobConfig, S3JobConfigTypedDict
from .scheduledtaskentitytype import ScheduledTaskEntityType
from .scheduledtaskinterval import ScheduledTaskInterval
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateScheduledTaskRequestTypedDict(TypedDict):
    connection_id: str
    entity_type: ScheduledTaskEntityType
    interval: ScheduledTaskInterval
    job_config: S3JobConfigTypedDict
    enabled: NotRequired[bool]


class DtoCreateScheduledTaskRequest(BaseModel):
    connection_id: str

    entity_type: ScheduledTaskEntityType

    interval: ScheduledTaskInterval

    job_config: S3JobConfig

    enabled: Optional[bool] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["enabled"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_entitlementsource import DtoEntitlementSource, DtoEntitlementSourceTypedDict
from .dto_featureresponse import DtoFeatureResponse, DtoFeatureResponseTypedDict
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoFeatureUsageSummaryTypedDict(TypedDict):
    current_usage: NotRequired[str]
    feature: NotRequired[DtoFeatureResponseTypedDict]
    is_enabled: NotRequired[bool]
    is_soft_limit: NotRequired[bool]
    is_unlimited: NotRequired[bool]
    next_usage_reset_at: NotRequired[str]
    sources: NotRequired[List[DtoEntitlementSourceTypedDict]]
    total_limit: NotRequired[int]
    usage_percent: NotRequired[str]


class DtoFeatureUsageSummary(BaseModel):
    current_usage: Optional[str] = None

    feature: Optional[DtoFeatureResponse] = None

    is_enabled: Optional[bool] = None

    is_soft_limit: Optional[bool] = None

    is_unlimited: Optional[bool] = None

    next_usage_reset_at: Optional[str] = None

    sources: Optional[List[DtoEntitlementSource]] = None

    total_limit: Optional[int] = None

    usage_percent: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "current_usage",
                "feature",
                "is_enabled",
                "is_soft_limit",
                "is_unlimited",
                "next_usage_reset_at",
                "sources",
                "total_limit",
                "usage_percent",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


SubscriptionStatus = Union[
    Literal[
        "active",
        "paused",
        "cancelled",
        "incomplete",
        "trialing",
        "draft",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionstatus import SubscriptionStatus
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoSubscriptionSummaryTypedDict(TypedDict):
    archived_at: NotRequired[str]
    r"""archived_at timestamp (for old subscriptions)"""
    billing_anchor: NotRequired[str]
    r"""billing_anchor of the subscription"""
    created_at: NotRequired[str]
    r"""created_at timestamp"""
    current_period_end: NotRequired[str]
    r"""current_period_end of the subscription"""
    current_period_start: NotRequired[str]
    r"""current_period_start of the subscription"""
    id: NotRequired[str]
    r"""id of the subscription"""
    plan_id: NotRequired[str]
    r"""plan_id of the subscription"""
    status: NotRequired[SubscriptionStatus]


class DtoSubscriptionSummary(BaseModel):
    archived_at: Optional[str] = None
    r"""archived_at timestamp (for old subscriptions)"""

    billing_anchor: Optional[str] = None
    r"""billing_anchor of the subscription"""

    created_at: Optional[str] = None
    r"""created_at timestamp"""

    current_period_end: Optional[str] = None
    r"""current_period_end of the subscription"""

    current_period_start: Optional[str] = None
    r"""current_period_start of the subscription"""

    id: Optional[str] = None
    r"""id of the subscription"""

    plan_id: Optional[str] = None
    r"""plan_id of the subscription"""

    status: Optional[SubscriptionStatus] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "archived_at",
                "billing_anchor",
                "created_at",
                "current_period_end",
                "current_period_start",
                "id",
                "plan_id",
                "status",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionstatus import SubscriptionStatus
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoUpdateSubscriptionRequestTypedDict(TypedDict):
    cancel_at: NotRequired[str]
    cancel_at_period_end: NotRequired[bool]
    parent_subscription_id: NotRequired[str]
    r"""ParentSubscriptionID sets or clears the parent subscription. Omit to leave unchanged; send \"\" to clear."""
    status: NotRequired[SubscriptionStatus]


class DtoUpdateSubscriptionRequest(BaseModel):
    cancel_at: Optional[str] = None

    cancel_at_period_end: Optional[bool] = None

    parent_subscription_id: Optional[str] = None
    r"""ParentSubscriptionID sets or clears the parent subscription. Omit to leave unchanged; send \"\" to clear."""

    status: Optional[SubscriptionStatus] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["cancel_at", "cancel_at_period_end", "parent_subscription_id", "status"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from flexprice_py.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetCustomerUsageSummaryRequestTypedDict(TypedDict):
    customer_id: NotRequired[str]
    customer_lookup_key: NotRequired[str]
    feature_ids: NotRequired[List[str]]
    feature_lookup_keys: NotRequired[List[str]]
    subscription_ids: NotRequired[List[str]]


class GetCustomerUsageSummaryRequest(BaseModel):
    customer_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    customer_lookup_key: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    feature_ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    feature_lookup_keys: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    subscription_ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "customer_id",
                "customer_lookup_key",
                "feature_ids",
                "feature_lookup_keys",
                "subscription_ids",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetCostAnalyticsRequestTypedDict(TypedDict):
    end_time: NotRequired[str]
    expand: NotRequired[List[str]]
    r"""Expand options - specify which entities to expand"""
    external_customer_id: NotRequired[str]
    r"""Optional - for specific customer"""
    feature_ids: NotRequired[List[str]]
    r"""Additional filters"""
    limit: NotRequired[int]
    r"""Pagination"""
    offset: NotRequired[int]
    start_time: NotRequired[str]
    r"""Time range fields (optional - defaults to last 7 days if not provided)"""


class DtoGetCostAnalyticsRequest(BaseModel):
    end_time: Optional[str] = None

    expand: Optional[List[str]] = None
    r"""Expand options - specify which entities to expand"""

    external_customer_id: Optional[str] = None
    r"""Optional - for specific customer"""

    feature_ids: Optional[List[str]] = None
    r"""Additional filters"""

    limit: Optional[int] = None
    r"""Pagination"""

    offset: Optional[int] = None

    start_time: Optional[str] = None
    r"""Time range fields (optional - defaults to last 7 days if not provided)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "end_time",
                "expand",
                "external_customer_id",
                "feature_ids",
                "limit",
                "offset",
                "start_time",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .walletconfigpricetype import WalletConfigPriceType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class WalletConfigTypedDict(TypedDict):
    allowed_price_types: NotRequired[List[WalletConfigPriceType]]
    r"""AllowedPriceTypes is a list of price types that are allowed for the wallet
    nil means all price types are allowed
    """


class WalletConfig(BaseModel):
    allowed_price_types: Optional[List[WalletConfigPriceType]] = None
    r"""AllowedPriceTypes is a list of price types that are allowed for the wallet
    nil means all price types are allowed
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["allowed_price_types"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
